@@ -123,17 +123,6 @@ class Device(DBusProxy):
         """Check if the device is external."""
         return not self.is_systeminternal
 
-    @property
-    def is_handleable(self):
-        """
-        Should this device be handled by udiskie?
-
-        Currently this just means that the device is removable and holds a
-        filesystem or the device is a LUKS encrypted volume.
-
-        """
-        return (self.is_filesystem or self.is_crypto) and self.is_external
-
     @property
     def is_mounted(self):
         """Check if the device is mounted."""
@@ -157,6 +146,11 @@ class Device(DBusProxy):
         """The filesystem path of the device block file."""
         return os.path.normpath(self.property.DeviceFile)
 
+    @property
+    def device_presentation(self):
+        """The device file path to present to the user."""
+        return self.property.DeviceFilePresentation
+
     @property
     def is_filesystem(self):
         return self.id_usage == 'filesystem'
@@ -284,16 +278,6 @@ class Udisks(DBusProxy):
             if dev.is_valid:
                 yield dev
 
-    def get_all_handleable(self):
-        """
-        Enumerate all handleable devices currently known to udisks.
-
-        NOTE: returns only devices that are still valid. This protects from
-        race conditions inside udiskie.
-
-        """
-        return (dev for dev in self.get_all() if dev.is_handleable)
-
     def get_device(self, path):
         """
         Get a device proxy by device name or any mount path of the device.
